From: Scott Carey <scott@richrelevance.com>
To: Miernik <public@public.miernik.name>
Cc: pgsql-performance@postgresql.org
Subject: Re: why query plan for the inner SELECT of WHERE x IN is wrong, but when run the inner query alone is OK?
Date: Sat, 9 Aug 2008 20:25:07 -0700
Message-ID: <a1ec7d000808092025ja0419f2ka2b1a323c36f439a@mail.gmail.com> (raw)
In-Reply-To: <20080810022059.7330.0.NOFFLE@turbacz.local>
References: <20080809203435.7227.0.NOFFLE@turbacz.local>
<20080809223208.7297.0.NOFFLE@turbacz.local>
<20080810022059.7330.0.NOFFLE@turbacz.local>
It is hardware dependent. The estimates are not time estimates, but on an
arbitrary scale.
On the server I work with, the estimates are almost always 10x larger than
the run times, and sometimes more than 50x.
(many GBs RAM, 8 CPU cores, more than 10 disks, standard optimizer settings
other than statistics sample sizes and increased common values for columns).
-Scott
On Sat, Aug 9, 2008 at 7:20 PM, Miernik <public@public.miernik.name> wrote:
> Miernik <public@public.miernik.name> wrote:
> > I present a SELECT uid plan with the 1000 table also below, just to be
> > sure, this is the "bad" plan, that takes forever:
> >
> > miernik=> EXPLAIN SELECT uid FROM cnts WHERE uid IN (SELECT uid FROM alog
> WHERE pid = 3452654 AND o = 1);
> > QUERY PLAN
> >
> -----------------------------------------------------------------------------------------------
> > Nested Loop IN Join (cost=0.00..3532.70 rows=1 width=4)
> > -> Seq Scan on cnts (cost=0.00..26.26 rows=1026 width=4)
> > -> Index Scan using alog_uid_idx on alog (cost=0.00..297.32 rows=1
> width=4)
> > Index Cond: ((alog.uid)::integer = (cnts.uid)::integer)
> > Filter: ((alog.pid = 3452654::numeric) AND (alog.o = 1::numeric))
> > (5 rows)
>
> If I reduce the number of rows in cnts to 100, I can actually make an
> EXPLAIN ANALYZE with this query plan finish in reasonable time:
>
> miernik=> EXPLAIN ANALYZE SELECT uid FROM cnts WHERE uid IN (SELECT uid
> FROM alog WHERE pid = 555949 AND odp = 1);
> QUERY
> PLAN
>
> -------------------------------------------------------------------------------------------------------------------------------------------------
> Nested Loop IN Join (cost=0.00..3585.54 rows=1 width=4) (actual
> time=51831.430..267844.815 rows=7 loops=1)
> -> Seq Scan on cnts (cost=0.00..14.00 rows=700 width=4) (actual
> time=0.005..148.464 rows=100 loops=1)
> -> Index Scan using alog_uid_idx on alog (cost=0.00..301.02 rows=1
> width=4) (actual time=2676.959..2676.959 rows=0 loops=100)
> Index Cond: ((alog.uid)::integer = (cnts.uid)::integer)
> Filter: ((alog.pid = 555949::numeric) AND (alog.odp = 1::numeric))
> Total runtime: 267844.942 ms
> (6 rows)
>
> The real running times are about 10 times more than the estimates. Is
> that normal?
>
> --
> Miernik
> http://miernik.name/
>
>
> --
> Sent via pgsql-performance mailing list (pgsql-performance@postgresql.org)
> To make changes to your subscription:
> http://www.postgresql.org/mailpref/pgsql-performance
>
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Reply to all the recipients using the --to and --cc options:
reply via email
To: pgsql-performance@postgresql.org
Cc: scott@richrelevance.com, public@public.miernik.name
Subject: Re: why query plan for the inner SELECT of WHERE x IN is wrong, but when run the inner query alone is OK?
In-Reply-To: <a1ec7d000808092025ja0419f2ka2b1a323c36f439a@mail.gmail.com>
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
This inbox is served by DDX for PostgreSQL; see mirroring instructions
for how to clone and mirror all data and code used for this inbox